-----------------------------------------------------------------------
-- Licensed Materials - Property of IBM
--
-- IBM Tape System Library Manager
--
-- (C) Copyright IBM Corp. 2004, 2014 All Rights Reserved
-----------------------------------------------------------------------

-- @version 1.1 - 14/02/13 22:10:31
--
-- @brief
-- CMVC   DATE         NAME   DESCRIPTIONS
--       (MM/DD/YY) 
-- ------ ---------- -------- ----------------------------------------
-- 74180  2014-02-14 mkamiya  Initial creation
--                            Replace following Triggers:
--                            DCA_INSERT
--                            SYSTEM_DCALIM_UPD
--                            

/
CREATE OR REPLACE PROCEDURE DELETE_DCA_RECORDS()
  LANGUAGE SQL
BEGIN
  DECLARE C1 INT;
  DECLARE C2 INT;
  DECLARE V1 TIMESTAMP;
  DECLARE SQLCODE INT DEFAULT 0;
  DECLARE csr CURSOR FOR
    SELECT RANK() OVER(ORDER BY TIMEUNMOUNT DESC) -1 AS COUNT, TIMEUNMOUNT FROM M_DRIVECARTRIDGEACCESS;
    SELECT MAX(SYSTEMDCALIMIT) INTO C2 FROM M_SYSTEM WHERE SYSTEMID <> 'INIT';
  OPEN csr;
  FETCH csr INTO C1, V1;
  WHILE (SQLCODE != 100) DO
    IF C1>=C2 AND V1 IS NOT NULL AND V1 <> '1900-01-01 00:00:00.000' THEN
      DELETE FROM M_DRIVECARTRIDGEACCESS WHERE CURRENT OF csr;
    END IF;
    FETCH csr INTO C1, V1;
  END WHILE;
  CLOSE csr;
END
/
CREATE or REPLACE TRIGGER SYSTEM_DCALIM_UPD AFTER UPDATE OF SYSTEMDCALIMIT ON M_SYSTEM
  FOR EACH STATEMENT MODE DB2SQL
  WHEN ((SELECT MAX(SYSTEMDCALIMIT) FROM M_SYSTEM WHERE SYSTEMID <> 'INIT') <= (SELECT COUNT(*) FROM M_DRIVECARTRIDGEACCESS))
BEGIN ATOMIC
  CALL DELETE_DCA_RECORDS();
END
/
CREATE or REPLACE TRIGGER DCA_INSERT AFTER INSERT ON M_DRIVECARTRIDGEACCESS
  FOR EACH STATEMENT MODE DB2SQL
  WHEN ((SELECT MAX(SYSTEMDCALIMIT) FROM M_SYSTEM WHERE SYSTEMID <> 'INIT') <= (SELECT COUNT(*) FROM M_DRIVECARTRIDGEACCESS))
BEGIN ATOMIC
  CALL DELETE_DCA_RECORDS();
END
/


-----------------------------------------------------------------------
