-----------------------------------------------------------------------
-- Licensed Materials - Property of IBM
--
-- IBM Tape System Library Manager
--
-- (C) Copyright IBM Corp. 2004, 2015 All Rights Reserved
-----------------------------------------------------------------------

-- @version 1.2 - 15/04/20 21:11:01
--
-- @brief
-- CMVC   DATE         NAME   DESCRIPTIONS
--       (MM/DD/YY) 
-- ------ ---------- -------- ----------------------------------------
-- 74180  2014-02-14 mkamiya  Initial creation
--                            Replace following Triggers:
--                            DCA_INSERT
--                            SYSTEM_DCALIM_UPD
-- 76933  2015-04-21 mkamiya  Bug fix
--                            


DROP TRIGGER SYSTEM_DCALIM_UPD/
DROP TRIGGER DCA_INSERT/
DROP PROCEDURE DELETE_DCA_RECORDS/

CREATE PROCEDURE DELETE_DCA_RECORDS()
  LANGUAGE SQL
  DISABLE DEBUG MODE
BEGIN

  DECLARE C1 INT;
  DECLARE C2 INT;
  DECLARE CT INT;
  DECLARE V1 TIMESTAMP;
  DECLARE ID CHAR(36);
  DECLARE SQLCODE INT DEFAULT 0;
  DECLARE SQLCODE1 INT DEFAULT 0;

  DECLARE csr CURSOR FOR
    SELECT RANK() OVER(ORDER BY TIMEUNMOUNT) AS COUNT, TIMEUNMOUNT, DRIVECARTRIDGEACCESSID FROM M_DRIVECARTRIDGEACCESS;
  SELECT MAX(SYSTEMDCALIMIT) INTO C2 FROM M_SYSTEM WHERE SYSTEMID <> 'INIT';
  OPEN csr;
  BEGIN
    DECLARE CONTINUE HANDLER FOR NOT FOUND
    BEGIN
      SET SQLCODE1 = 100;
    END;
    DECLARE EXIT HANDLER FOR SQLEXCEPTION
    BEGIN
      SET SQLCODE1 = SQLCODE;
      RETURN SQLCODE1;
    END;
    FETCH csr INTO C1, V1, ID;
    SELECT COUNT(*) INTO CT FROM M_DRIVECARTRIDGEACCESS;
    WHILE (SQLCODE <> 100 AND CT>C2) DO
      IF V1 IS NOT NULL AND V1 <> '1900-01-01 00:00:00.000' THEN
        DELETE FROM M_DRIVECARTRIDGEACCESS WHERE M_DRIVECARTRIDGEACCESS.DRIVECARTRIDGEACCESSID=ID;
      END IF;
      FETCH csr INTO C1, V1, ID;
      SELECT COUNT(*) INTO CT FROM M_DRIVECARTRIDGEACCESS;
    END WHILE;
  END;
  CLOSE csr;
  RETURN 0;
END
/

CREATE TRIGGER SYSTEM_DCALIM_UPD AFTER UPDATE OF SYSTEMDCALIMIT ON M_SYSTEM
  FOR EACH STATEMENT MODE DB2SQL
  WHEN ((SELECT MAX(SYSTEMDCALIMIT) FROM M_SYSTEM WHERE SYSTEMID <> 'INIT') <= (SELECT COUNT(*) FROM M_DRIVECARTRIDGEACCESS))
BEGIN ATOMIC
  CALL DELETE_DCA_RECORDS();
END
/
CREATE TRIGGER DCA_INSERT AFTER INSERT ON M_DRIVECARTRIDGEACCESS
  FOR EACH STATEMENT MODE DB2SQL
  WHEN ((SELECT MAX(SYSTEMDCALIMIT) FROM M_SYSTEM WHERE SYSTEMID <> 'INIT') <= (SELECT COUNT(*) FROM M_DRIVECARTRIDGEACCESS))
BEGIN ATOMIC
  CALL DELETE_DCA_RECORDS();
END
/


-----------------------------------------------------------------------
