#!/bin/sh
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2017 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-----------------------------------------------------------------------
#
# @version 1.8 - 17/05/11 17:51:32
#
# @brief tslm_install - Install TSLM on xlinux/aix platforms. (82229)
#
#-----------------------------------------------------------------------
export LC_ALL=C

########################################################################
# variable definition
########################################################################
# log functions
LOG_ROOT="/tmp/tslmlog"
TSLM_INST_LOG_FILE_NAME="$LOG_ROOT/tslm_inst.log"
TSLM_INST_STAT_FILE_NAME="$LOG_ROOT/tslm_inst_stat"
TSLM_UNINST_LOG_FILE_NAME="$LOG_ROOT/tslm_uninst.log"

# product informatikon
PRODUCT_NAME="Tape System Library Manager"
PRODUCT_SHORT="TSLM"
TSLM_USR_NAME="ermm"
TSLM_DB_NAME="ermm"
TSLM_SU_COMMAND="su - $TSLM_USR_NAME -c"

INSTALLATION_BASE_DIR="/opt/IBM/ermm"
INSTALLATION_BIN_DIR="$INSTALLATION_BASE_DIR/bin"
INSTALLATION_INSTALL_DIR="$INSTALLATION_BASE_DIR/install"
INSTALLATION_CONFIG_DIR="$INSTALLATION_BASE_DIR/conf"
INSTALLATION_TSM_DIR="$INSTALLATION_BASE_DIR/client/tsm"
INSTALLATION_EMU3494_DIR="$INSTALLATION_BASE_DIR/client/3494"
INSTALLATION_LOG_DIR="$INSTALLATION_BASE_DIR/log"

PRE_INSTALL_DB_SCRIPT="preInstallDatabase"
INSTALL_DB_SCRIPT="installDatabase"

MM_CONFIG_FILE_SAMPLE="mm.xml.smp"
MM_CONFIG_FILE="mm.xml"
HDM_CONFIG_FILE_SAMPLE="genhdm.cfg.smp"
HDM_CONFIG_FILE="genhdm.cfg"
ELM_CONFIG_FILE_SAMPLE="elm.cfg.smp"
ELM_CONFIG_FILE="elm.cfg"
EMU3494_CONFIG_FILE_SAMPLE="ermm3494.cfg.smp"
EMU3494_CONFIG_FILE="ermm3494.cfg"


SCRIPT_NAME=""
SRCDIR=`dirname $0`
CURRENT_DIR=`pwd`
TSLM_INST_PKG_ROOT="$SRCDIR/../"
cd $TSLM_INST_PKG_ROOT
TSLM_INST_PKG_ROOT=`pwd`
cd $CURRENT_DIR
TSLM_INST_SCRIPT_DIR="$TSLM_INST_PKG_ROOT/scripts"
PACKAGE_FILES_DIR=""
TSLM_SYS_INSTALLED="false"


# application related
TSLM_PKG_CLI_KEY="cli"
TSLM_PKG_ELM_KEY="elm"
TSLM_PKG_HDM_KEY="hdm"
TSLM_PKG_LM_KEY="lm"
TSLM_PKG_MM_KEY="mm"
TSLM_PKG_PWGEN_KEY="pwgen"
TSLM_PKG_SYSTEM_KEY="system"
TSLM_PKG_3494_KEY="3494"
TSLM_PKG_LIBBAL_KEY="libbal"

TSLM_STOP_SCRIPT="ermmStop"
TSLM_LM_STOP_SCRIPT="ermmLmStop"
TSLM_GEN_HDM_SCRIPT="ermmGenHDM"
TSLM_MM_BIN="MediaManager"
TSLM_LM_BIN="LibraryManager"

# other components required
PERL_NAME="Perl"
PERL_FILESET_LINUX="perl"
PERL_FILESET_AIX="perl.rte"
PERL_REQ_VRMF="5.8.0.0"

TAPE_DRIVER_NAME_LINUX="lin_tape"
TAPE_DRIVER_FILESET_LINUX="lin_tape"
TAPE_DRIVER_REQ_VRMF_LINUX="1.74.0.0"
TAPE_DRIVER_NAME_AIX="Atape"
TAPE_DRIVER_FILESET_AIX="Atape.driver"
TAPE_DRIVER_REQ_VRMF_AIX="12.5.2.0"

DB_NAME="DB2"
DB_ADM_NAME="db2inst1"
DB_ADM_GRP_NAME="db2grp1"
DB_VER=""
DB_ADM_DIR="/home/db2inst1"
SQLLIB_PATH="/home/db2inst/sqllib/bin"
DB2_INSTALL_LOG="ermmdb2install.log"


# process state
NUM_OF_UNINSTALLED_PKG=0

# recognize the platform (check_platform)
MACHINE_REQ_AIX="6.1"               # version 6 or 7 (AIX/RHEL/SLES)
MACHINE_REQ_SLES="11.4"             # version 6 or 7 (AIX/RHEL/SLES)
MACHINE_REQ_RHEL="6.1"              # version 6 or 7 (AIX/RHEL/SLES)

# TSLM configuration file
TSLM_CONF_FILE="/etc/tslm.cfg"

# System Information
MACHINE_OS=""                # AIX or RHEL (uname/ /etc/redhat-release)
MACHINE_CPU=""               # x86_64 or powerpc (uname -p)
MACHINE_VER=""               # version 6 or 7 (AIX/RHEL/SLES)
MACHINE_REL=""               # release 1,2,... (AIX/RHEL/SLES)

# variables for compatible mode
CMP_MODE=""                  # install_xlinux.sh, uninstall_xlinux.sh
                             # install_aix.sh, uninstall_aix.sh

# install options
# Native Command Options
OPT_MM="false"               # option of Media Manager
OPT_LM="false"               # option of Library Manager
OPT_LB="false"               # option of Library Balancer
OPT_SYS="false"              # option of System
OPT_3494="false"             # option of 3494 emulator
OPT_ELM="false"              # option of External Library Manager
OPT_HDM="false"              # option of Host Driver Manager
OPT_PW="false"               # option of Password Generator
OPT_CLI="false"              # option of CLI
OPT_DROPDB="false"           # drop database

OPT_INSTALL="false"          # installation mode
OPT_UNINSTALL="false"        # uninstallation mode

OPT_CLIENT="false"           # client installation
OPT_SERVER="false"           # server installation
OPT_BOTH="false"             # both option
OPT_EMU="false"              # 3494 emulation
OPT_PRODUCT="false"          # uninstall product only
OPT_ONLY_CHK_PREREQ="false"  # check prereq only

OPT_HELP="false"             # show usage


# Compatible Mode Options
COPT_CMP="false"             # is compatible mode?
COPT_CLIENT="false"          # client installation
COPT_SERVER="false"          # server installation
COPT_BOTH="false"            # both option
COPT_EMU="false"             # 3494 emulation
COPT_PRODUCT="false"         # uninstall product only
COPT_DROPDB="false"          # drop database
COPT_HELP="false"            # show usage

########################################################################
# system functions
########################################################################
#
# provide information
#
show_copyright()
{
    echo "IBM Tape System Library Manager"
    echo "Licensed Materials - Property of IBM"
    echo "(C) Copyright IBM Corp. 2004, 2017 All Rights Reserved"
    echo ""
    sleep 1
}

usage()
{
    local rc=$1
    if [[ $rc != 0 && $COPT_CMP == "true" ]]; then
	COPT_HELP="true"
    elif [[ $rc != 0 ]]; then
	OPT_HELP="true"
    fi

    if [[ $COPT_CMP == "true" && $COPT_HELP == "true" ]]; then # compatible mode
	if [[ $CMP_MODE == "install_xlinux.sh" ]]; then
	    echo "Install $PRODUCT_NAME and create database $TSLM_DB_NAME."
	    echo "Usage: $(basename $CMP_MODE) [-c] [-s] [-b] [-e] [-h]"
	    echo "Options:"
	    echo "  -c    Install client packages of $PRODUCT_SHORT."
	    echo "  -s    Install server packages of $PRODUCT_SHORT."
	    echo "  -b    Install both, client and server packages of $PRODUCT_SHORT."
	    echo "  -e    Install 3494 emulator package. Must be specified with -c/-s/-b."
	    echo "  -h    Display this help and exit."
	elif [[ $CMP_MODE == "uninstall_xlinux.sh" ]]; then
	    echo "Uninstall $PRODUCT_NAME and drop database $TSLM_DB_NAME."
	    echo "Usage: $(basename $CMP_MODE) [-p] [-d] [-b] [-h]"
	    echo "Options:"
	    echo "  -p    Uninstall $PRODUCT_SHORT packages only."
	    echo "  -d    Drop database $TSLM_DB_NAME only."
	    echo "  -b    Uninstall $PRODUCT_SHORT packages and drop database $TSLM_DB_NAME."
	    echo "  -h    Display this help and exit."
	elif [[ $CMP_MODE == "install_aix.sh" ]]; then
	    echo "Install $PRODUCT_NAME and create database $TSLM_DB_NAME."
	    echo "Usage: $(basename $CMP_MODE) [-c] [-s] [-b] [-e] [-h]"
	    echo "Options:"
	    echo "  -c    Install client packages of $PRODUCT_SHORT."
	    echo "  -s    Install server packages of $PRODUCT_SHORT."
	    echo "  -b    Install both, client and server packages of $PRODUCT_SHORT."
	    echo "  -e    Install 3494 emulator package. Must be specified with -c/-s/-b."
	    echo "  -h    Display this help and exit."
	elif [[ $CMP_MODE == "uninstall_aix.sh" ]]; then
	    echo "Uninstall $PRODUCT_NAME and drop database $TSLM_DB_NAME."
	    echo "Usage: $(basename $CMP_MODE) [-p] [-d] [-b] [-h]"
	    echo "Options:"
	    echo "  -p    Uninstall $PRODUCT_SHORT packages only."
	    echo "  -d    Drop database $TSLM_DB_NAME only."
	    echo "  -b    Uninstall $PRODUCT_SHORT packages and drop database $TSLM_DB_NAME."
	    echo "  -h    Display this help and exit."
	else
	    echo "Unexpected Error!"
	fi
	exit $rc
    elif [[ $OPT_HELP == "true" ]]; then  # native mode
	echo "Install/Uninstall $PRODUCT_NAME and create/drop database $TSLM_DB_NAME."
	echo "Usage: $SCRIPT_NAME [-i [csbeMLBS3EDPCI]] [-u [pbdMLBS3EDPCI]] [-h]"
	echo "Options:"
	echo "  -i    Install $PRODUCT_SHORT packages ."
	echo "  -u    Uninstall $PRODUCT_SHORT packages ."
	echo "  -c    Install client packages of $PRODUCT_SHORT."
	echo "  -s    Install server packages of $PRODUCT_SHORT."
	echo "  -b    Install both, client and server packages of $PRODUCT_SHORT."
	echo "        Uninstall $PRODUCT_SHORT packages and drop database $TSLM_DB_NAME."
        echo "  -p    Uninstall Product only."
	echo "  -d    Drop database $TSLM_DB_NAME only."
	echo "  -M    Install/Uninstall Media Manager."
	echo "  -L    Install/Uninstall Library Manager."
	echo "  -B    Install/Uninstall Library Balancer."
	echo "  -S    Install/Uninstall $PRODUCT_SHORT Base System."
	echo "  -3    Install/Uninstall 3494 Emulator."
	echo "  -E    Install/Uninstall External Library Manager."
	echo "  -D    Install/Uninstall Host Drive Manager."
	echo "  -P    Install/Uninstall Password Creation Tool."
	echo "  -C    Install/Uninstall CLI."
        echo "  -I    Check prerequisite components. (only)"
	echo "  -h    Display this help and exit."
	exit $rc
    fi
}

#
# check user
#
check_user()
{
    if [[ $USER != "root" ]]; then
        echo "This script can not be run with non-root account."
        exit 1
    fi
}

#
# log functions
#
print_and_log()
{
    echo $1 | tee -a $TSLM_INST_LOG_FILE_NAME
}

logonly()
{
    echo $1 >> $TSLM_INST_LOG_FILE_NAME
}

create_log_file()
{
    if [[ ! -d $LOG_ROOT ]]; then
        echo "Create log directory '$LOG_ROOT'."
        mkdir -p $LOG_ROOT
        if [[ $? != 0 ]]; then
            echo "Failed to create log directory '$LOG_ROOT'."
            exit 2
        fi
    fi
    
    if [[ -f $TSLM_INST_LOG_FILE_NAME ]]; then
        mv $TSLM_INST_LOG_FILE_NAME "$TSLM_INST_LOG_FILE_NAME"'.previous'
    fi

    touch $TSLM_INST_LOG_FILE_NAME
}

banner()
{
    BANNER=""
    width=$(tput cols)
    if [[ $width -le 80 ]]; then
        i=0
        while [[ $i -lt $width ]]; do
            BANNER="$BANNER#"
            let i=$i+1
        done
    else
        BANNER="###############################################################################"
    fi
    print_and_log "$BANNER"
}

print_and_log_with_banner()
{
    banner
    print_and_log "$1"
    banner
}

ask_yes_no_answer()
{
    while true;
    do
	print_and_log "$1 (y/n)"
	read YES_NO_ANS
	YES_NO_ANS=`echo $YES_NO_ANS | grep -iE '^(yes|y|no|n)$' | tr "[:lower:]" "[:upper:]"`
	case "$YES_NO_ANS" in
	    "Y" | "YES") return 1;;
	    "N" | "NO") return 0;;
	esac
    done
    logonly $YES_NO_ANS
}

#
# check platform
#
# MACHINE_OS, MACHINE_CPU, MACHINE_VER, MACHINE_REL
#
#
check_platform()
{
    # AIX? Linux?
    _uname=`uname`
    if [[ $_uname == "AIX" ]]; then
	MACHINE_OS="AIX"
	MACHINE_CPU=`uname -p`
	MACHINE_VER=`uname -v`
	MACHINE_REL=`uname -r`
    elif [[ $_uname == "Linux" ]]; then
	if [[ -e "/etc/SuSE-release" ]]; then
	    MACHINE_OS="SLES"
	    MACHINE_CPU=`uname -p`
	    MACHINE_VER=`cat /etc/SuSE-release | grep VERSION | tr -d ' ' | cut -f2 -d'='`
            MACHINE_REL=`cat /etc/SuSE-release | grep PATCHLEVEL | tr -d ' ' | cut -f2 -d'='`
        elif [[ -e "/etc/redhat-release" ]]; then
	    MACHINE_OS="RHEL"
	    MACHINE_CPU=`uname -p`
	    MACHINE_VER=`cat /etc/redhat-release | awk '/release/{ if(match($0, /release.+[0-9]+\.[0-9]+/)) print substr($0, RSTART, RLENGTH);}'|cut -f2 -d' '|cut -f1 -d'.'`
	    MACHINE_REL=`cat /etc/redhat-release | awk '/release/{ if(match($0, /release.+[0-9]+\.[0-9]+/)) print substr($0, RSTART, RLENGTH);}'|cut -f2 -d' '|cut -f2 -d'.'`
        else
            print_and_log "this OS is not supported"
	    exit 3
        fi

    else
	print_and_log "this OS($_uname) is not supported"
	exit 4
    fi
    print_and_log "$MACHINE_OS($MACHINE_CPU) $MACHINE_VER.$MACHINE_REL detected"
}

#
# check options
#
check_options()
{
    SCRIPT_NAME=$0
    while getopts "MLBS3EDPCIiuhx:csbepd" OPT ; do
	case $OPT in
	    x)  #compatible mode
                CMP_MODE=`basename $OPTARG`
		COPT_CMP="true"
		print_and_log "compatible mode:$CMP_MODE"
		if [[ $CMP_MODE == "install_xlinux.sh" || $CMP_MODE == "install_aix.sh" ]]; then
		    OPT_INSTALL="true"
		elif [[ $CMP_MODE == "uninstall_xlinux.sh" || $CMP_MODE == "uninstall_aix.sh" ]]; then
		    OPT_UNINSTALL="true"
		fi
		;;
	    c)  #client installation
		COPT_CLIENT="true"
		OPT_CLIENT="true"
		;;
	    s)  #server installation
		COPT_SERVER="true"
		OPT_SERVER="true"
		;;
	    b)  #both mode (used by client/server for different purposes
		COPT_BOTH="true"
		OPT_BOTH="true"
		;;
	    e)  #installation of 3494 emulation
		COPT_EMU="true"
		OPT_EMU="true"
		;;
	    p)  #product only
		COPT_PRODUCT="true"
		OPT_PRODUCT="true"
		;;
	    d)  #drop DB
		COPT_DROPDB="true"
		OPT_DROPDB="true"
		;;

	    #
	    # Native Command Options
	    #
	    i)  #installation mode
		OPT_INSTALL="true"
		;;
	    u)  #uninstallation mode
		OPT_UNINSTALL="true"
		;;
	    M)  #Media Manager
		OPT_MM="true"
		;;
	    L)  #Library Manager
		OPT_LM="true"
		;;
	    B)  #Library Manager Balancer
		OPT_LB="true"
		;;
	    S)  #System
		OPT_SYS="true"
		;;
	    3)  #3494 Emulator
		OPT_3494="true"
		;;
	    E)  #External Library Manager
		OPT_ELM="true"
		;;
	    D)  #Host Driver Manager
		OPT_HDM="true"
		;;
	    P)  #Password Generator
		OPT_PW="true"
		;;
	    C)  #CLI
		OPT_CLI="true"
		;;
	    I)  #check prereq only
		OPT_ONLY_CHK_PREREQ="true"
		;;
	    h)
		COPT_HELP="true"
		OPT_HELP="true"
		;;
	    \?)
		echo "Invalid option was specified:"
		usage 4
		;;
	esac
    done
}

#
# validity check of options
#
check_valid_options()
{
    # convert parameters from compatible mode to native mode
    if [[ $OPT_INSTALL == "true" && $OPT_UNINSTALL == "true" ]]; then
	echo "Invalid option was specified:"
	usage 5
    elif [[ $OPT_INSTALL == "false" && $OPT_UNINSTALL == "false" ]]; then
	echo "Invalid option was specified:"
	usage 6
    elif [[ $CMP_MODE == "install_xlinux.sh" || $CMP_MODE == "install_aix.sh" || $OPT_INSTALL == "true" ]]; then
	OPT_INSTALL="true"
	if [[ $COPT_CLIENT == "true" || $OPT_CLIENT == "true" || $COPT_BOTH == "true" || $OPT_BOTH == "true" ]]; then
	    OPT_ELM="true"
	    OPT_HDM="true"
	fi
	if [[ $COPT_SERVER == "true" || $OPT_SERVER == "true" || $COPT_BOTH == "true" || $OPT_BOTH == "true" ]]; then
	    OPT_MM="true"
	    OPT_LM="true"
	    OPT_CLI="true"
        OPT_SYS="true" 
	fi
	if [[ $COPT_EMU == "true" || $OPT_EMU == "true" ]]; then
	    OPT_3494="true"
	fi
	if [[ $COPT_CLIENT == "true" || $OPT_CLIENT == "true" || $COPT_BOTH == "true" || $OPT_BOTH == "true" ]]; then	
	    OPT_SYS="true"
	    OPT_CLI="true"
	    OPT_PW="true"
	fi
    elif [[ $CMP_MODE == "uninstall_xlinux.sh" || $CMP_MODE == "uninstall_aix.sh" || $OPT_UNINSTALL == "true" ]]; then
	OPT_UNINSTALL="true"
	if [[ $COPT_PRODUCT == "true" || $OPT_PRODUCT == "true" || $COPT_BOTH == "true" || $OPT_BOTH == "true" ]]; then
	    OPT_MM="true"
	    OPT_LM="true"
	    OPT_3494="true"
	    OPT_ELM="true"
	    OPT_HDM="true"
	    OPT_PW="true"
	    OPT_CLI="true"
	    OPT_SYS="true"
	    OPT_LB="true"
	    OPT_PRODUCT="true"
	fi
	if [[ $COPT_DROPDB == "true" || $COPT_BOTH == "true" || $OPT_BOTH == "true" ]]; then
	    if [[ $OPT_PRODCUT != "true" ]]; then
		OPT_DROPDB="true"
	    fi
	fi
    fi

    if [[ $OPT_INSTALL == "true" ]]; then
	if [[ $OPT_DROPDB == "true" || $OPT_PRODUCT == "true" ]]; then
	    echo "Invalid option was specified:"
	    usage 7 
	fi
    elif [[ $OPT_UNINSTALL == "true" ]]; then
	if [[ $OPT_CLIENT == "true" || $OPT_SERVER == "true" || $OPT_EMU == "true" ]]; then
	    echo "Invalid option was specified:"
	    usage 8
	fi
    fi

    if [[ $OPT_INSTALL == "true" ]]; then
	if [[ $OPT_SYS != "true" && $OPT_CLI != "true" && $OPT_MM != "true" && $OPT_LM != "true" && $OPT_3494 != "true" && $OPT_ELM != "true" && $OPT_HDM != "true" && $OPT_LB != "true" && $OPT_PW != "true" ]]; then
	    usage 30
	fi
    elif [[ $OPT_UNINSTALL == "true" ]]; then
	if [[ $OPT_SYS != "true" && $OPT_CLI != "true" && $OPT_MM != "true" && $OPT_LM != "true" && $OPT_3494 != "true" && $OPT_ELM != "true" && $OPT_HDM != "true" && $OPT_LB != "true" && $OPT_PW != "true" && $OPT_DROPDB != "true" ]]; then
	    usage 31
	fi
    fi


}

#
# show menu of installation/uninstallation
#
show_action_menu()
{
    if [[ $OPT_INSTALL == "true" ]]; then
	print_and_log_with_banner "$PRODUCT_SHORT Installation Menu"
	if [[ $OPT_SYS == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT System"
	fi
	if [[ $OPT_MM == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Media Manager"
	fi
	if [[ $OPT_LM == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Library Manager"
	fi
	if [[ $OPT_LB == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Library Balancer"
	fi
	if [[ $OPT_ELM == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT External Library Manager"
	fi
	if [[ $OPT_HDM == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Host Drive Manager"
	fi
	if [[ $OPT_CLI == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT CLI"
	fi
	if [[ $OPT_3494 == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT 3494 Emulator"
	fi
	if [[ $OPT_PW == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Password Creation Tool (pwgen)"
	fi

    elif [[ $OPT_UNINSTALL == "true" ]]; then
	print_and_log_with_banner "$PRODUCT_SHORT Uninstallation Menu"
	if [[ $OPT_SYS == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT System"
	fi
	if [[ $OPT_MM == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Media Manager"
	fi
	if [[ $OPT_LM == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Library Manager"
	fi
	if [[ $OPT_LB == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Library Balancer"
	fi
	if [[ $OPT_ELM == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT External Library Manager"
	fi
	if [[ $OPT_HDM == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Host Drive Manager"
	fi
	if [[ $OPT_CLI == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT CLI"
	fi
	if [[ $OPT_3494 == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT 3494 Emulator"
	fi
	if [[ $OPT_PW == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT Password Creation Tool (pwgen)"
	fi
	if [[ $OPT_DROPDB == "true" ]]; then
	    print_and_log ""
	    print_and_log "Caution! '$TSLM_DB_NAME' database will be dropped"
	fi
    fi
    
    ask_yes_no_answer "Would do you like to continue? " && print_and_log "Aborted!" && exit 9
}

#
# stop or kill TSLM server processes
#
stop_mm_by_script()
{
    if [ -a $INSTALLATION_BIN_DIR/$TSLM_STOP_SCRIPT ]; then
        print_and_log "Trying to run $INSTALLATION_BIN_DIR/$TSLM_STOP_SCRIPT..."
        $TSLM_SU_COMMAND "$INSTALLATION_BIN_DIR/$TSLM_STOP_SCRIPT"
	print_and_log ""
    fi

}

stop_lm_by_script()
{
    if [ -a $INSTALLATION_BIN_DIR/$TSLM_LM_STOP_SCRIPT ]; then
        print_and_log "Trying to run $INSTALLATION_BIN_DIR/$TSLM_LM_STOP_SCRIPT..."
        $TSLM_SU_COMMAND "$INSTALLATION_BIN_DIR/$TSLM_LM_STOP_SCRIPT"
        $TSLM_SU_COMMAND "$INSTALLATION_BIN_DIR/$TSLM_LM_STOP_SCRIPT -f"
	print_and_log ""
    fi

}

kill_mm_process()
{
    PS_MM_PID=`ps aux | grep $INSTALLATION_BIN_DIR/$TSLM_MM_BIN | grep -v grep | awk '{print $2}'`
    logonly "PS_MM_PID=$PS_MM_PID"

    if [ -n "$PS_MM_PID" ]; then
        print_and_log "Killing process of $TSLM_MM (PID: $PS_MM_PID)..."
        KILL_MM_PID=`kill -9 $PS_MM_PID`
        logonly "KILL_MM_PID=$KILL_MM_PID"
    fi

    print_and_log ""
}

kill_lm_process()
{
    PS_LM_PID=`ps aux | grep $INSTALLATION_BIN_DIR/$TSLM_LM_BIN | grep -v grep | head -n 1 | awk '{print $2}'`
    logonly "PS_LM_PID=$PS_LM_PID"

    while [ -n "$PS_LM_PID" ]; do
        print_and_log "Killing process of $TSLM_LM (PID: $PS_LM_PID)..."
        KILL_LM_PID=`kill -9 $PS_LM_PID`
        logonly "KILL_LM_PID=$KILL_LM_PID"
        PS_LM_PID=`ps aux | grep $INSTALLATION_BIN_DIR/$TSLM_LM_BIN | grep -v grep | head -n 1 | awk '{print $2}'`
        logonly "PS_LM_PID=$PS_LM_PID"
    done

    print_and_log ""
}

kill_hdm_process()
{
    PS_HDM_PID=`ps aux | grep $INSTALLATION_BIN_DIR/$TSLM_GEN_HDM_SCRIPT | grep -v grep | awk '{print $2}'`
    logonly "PS_HDM_PID=$PS_HDM_PID"

    if [ -n "$PS_HDM_PID" ]; then
        print_and_log "Killing process of $TSLM_HDM (PID: $PS_HDM_PID)..."
        KILL_HDM_PID=`kill -9 $PS_HDM_PID`
        logonly "KILL_HDM_PID=$KILL_HDM_PID"
	print_and_log ""
    fi

}

quit_install_uninstall()
{
    print_and_log "Install/Uninstall process of $PRODUCT_SHORT did not complete successfully: RC=$1"
    print_and_log ""
    print_and_log_with_banner "End of $PRODUCT_SHORT install/uninstall process on `date`"
    exit $1
}

remove_pkg()
{
    local Q_NUM=""
    local REMOVE_RC=""
    if [[ $MACHINE_OS == "AIX" ]]; then   #AIX
	local target="tslm.$1"
	lslpp -l $target
	Q_NUM=$?
	if [[ $Q_NUM -eq 0 ]]; then
            echo "Attempting to remove $target"
            installp -u -X $target 1>>$TSLM_UNINST_LOG_FILE_NAME 2>&1
            REMOVE_RC=$?
            if [[ $REMOVE_RC -eq 0 ]]; then
		NUM_OF_UNINSTALLED_PKG=`expr $NUM_OF_UNINSTALLED_PKG + 1`
		print_and_log "Removal of $target was successful."
	    elif [[ $REMOVE_RC -eq 255 ]]; then
		installp -C $target 1>>$TSLM_UNINST_LOG_FILE_NAME 2>&1
		REMOVE_RC=$?
		if [[ $REMOVE_RC -eq 0 ]]; then
                    NUM_OF_UNINSTALLED_PKG=`expr $NUM_OF_UNINSTALLED_PKG + 1`
                    print_and_log "Removal of $target was successful."
		else
                    print_and_log "Failed to remove $target: RC=$REMOVE_RC"
                    quit_install_uninstall 11
		fi
            else
		print_and_log "Failed to remove $target: RC=$REMOVE_RC"
		quit_install_uninstall 11
            fi
            print_and_log ""
	fi
    else   #Linux
	local target="tslm-$1"
	rpm -q $target
	Q_NUM=$?
	if [[ $Q_NUM -eq 0 ]]; then
            echo "Attempting to remove $target"
            rpm -ev $target 1>>$TSLM_UNINST_LOG_FILE_NAME 2>&1
            REMOVE_RC=$?
            if [[ $REMOVE_RC -eq 0 ]]; then
		NUM_OF_UNINSTALLED_PKG=`expr $NUM_OF_UNINSTALLED_PKG + 1`
		print_and_log "Removal of $target was successful."
            else
		print_and_log "Failed to remove $target: RC=$REMOVE_RC"
		quit_install_uninstall 11
            fi
            print_and_log ""
	fi
    fi
}

set_package_dir()
{
    if [[ $MACHINE_OS == "AIX" ]]; then
	PACKAGE_FILES_DIR="$TSLM_INST_PKG_ROOT/aix64"
    elif [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "7" ]]; then
	PACKAGE_FILES_DIR="$TSLM_INST_PKG_ROOT/linux_r7_64"
    else
	PACKAGE_FILES_DIR="$TSLM_INST_PKG_ROOT/linux64"
    fi
}

install_pkg()
{
    local INSTALL_RC=""
    local target=`ls $PACKAGE_FILES_DIR | grep "\-$1\-"`
    logonly "Target package: $1, Found package: $target"
    if [[ -z "$target" ]]; then
            print_and_log "\"$1\" package is not found in \"$PACKAGE_FILES_DIR\" directory."
            print_and_log "This package is required to proceed installation."
            quit_install_uninstall 12
    fi
    print_and_log "Attempting to install $target"
    if [[ $MACHINE_OS == "AIX" ]]; then
	installp -acgNQqXYd "$PACKAGE_FILES_DIR/$target" all 1>>$TSLM_INST_LOG_FILE_NAME 2>&1
    else
	rpm -ivh "$PACKAGE_FILES_DIR/$target" 1>>$TSLM_INST_LOG_FILE_NAME 2>&1
    fi
    INSTALL_RC=$?
    if [ $INSTALL_RC -eq 0 ]; then
        print_and_log "Installation of $1 was successful."
    else
        print_and_log "Failed to install $1: RC=$INSTALL_RC"
        quit_install_uninstall 12
    fi
    print_and_log ""
}

#
# check os version
#
check_os_version()
{
    print_and_log "Checking OS Version..."
    print_and_log "$MACHINE_OS($MACHINE_CPU) $MACHINE_VER.$MACHINE_REL"
    if [[ $MACHINE_OS == "AIX" ]]; then #AIX
	compare_vrmfs "$MACHINE_VER.$MACHINE_REL.0.0" "$MACHINE_REQ_AIX.0.0"
    elif [[ $MACHINE_OS == "SLES" ]]; then #SLES
	if [[ $OPT_MM == "true" || $OPT_LM == "true" ]]; then
	    print_and_log "SLES can't work as TSLM Server"
	    return 16
	fi
	compare_vrmfs "$MACHINE_VER.$MACHINE_REL.0.0" "$MACHINE_REQ_SLES.0.0"
    elif [[ $MACHINE_OS == "RHEL" ]]; then #RHEL
	compare_vrmfs "$MACHINE_VER.$MACHINE_REL.0.0" "$MACHINE_REQ_RHEL.0.0"
    else
	print_and_log "TSLM doesn't support this operating system"
	return 16
    fi
    if [ $? -eq "2" ]; then
	print_and_log "version of this operating system is old."
	return 16
    fi
    print_and_log "Accepted"
    print_and_log ""
    return 0
}


#
# check if it's installed and ask if remove it
#
check_previous_installation()
{
    local target=$1
    local msg=""
    local rc=""

    if [[ $MACHINE_OS == "AIX" ]]; then
	msg=`lslpp -l tslm.$target`
	rc=$?
    else
	msg=`rpm -q tslm-$target`
	rc=$?	
    fi
    if [[ $rc == 0 && $target != $TSLM_PKG_SYSTEM_KEY ]]; then
	print_and_log "'$target' is installed already as below."
	print_and_log "$msg"
	print_and_log "In order to continue this installation process, uninstallation of '$target' is required first."
	ask_yes_no_answer "Would you like to uninstall it? " && print_and_log "Aborted!" && exit 10
	if [[ $target == $TSLM_PKG_MM_KEY ]]; then
	    stop_mm_by_script
	    kill_mm_process
	    print_and_log "Done!"
	    print_and_log ""
	elif [[ $target == $TSLM_PKG_LM_KEY ]]; then
	    stop_lm_by_script
	    kill_lm_process
	    print_and_log "Done!"
	    print_and_log ""
	elif [[ $target == $TSLM_PKG_HDM_KEY ]]; then
	    kill_hdm_process
	    print_and_log "Done!"
	    print_and_log ""	    
	fi
	
	remove_pkg $target
    elif [[ $rc == 0 && $target == $TSLM_PKG_SYSTEM_KEY ]]; then
	TSLM_SYS_INSTALLED="true"
    fi
}

check_previous_installations()
{
    if [[ $OPT_INSTALL == "true" ]]; then
	print_and_log_with_banner "Checking previous installations..."
	if [[ $OPT_SYS == "true" ]]; then
	    check_previous_installation $TSLM_PKG_SYSTEM_KEY
	fi
	if [[ $OPT_MM == "true" ]]; then
	    check_previous_installation $TSLM_PKG_MM_KEY
	fi
	if [[ $OPT_LM == "true" ]]; then
	    check_previous_installation $TSLM_PKG_LM_KEY
	fi
	if [[ $OPT_LB == "true" ]]; then
	    check_previous_installation $TSLM_PKG_LIBBAL_KEY
	fi
	if [[ $OPT_ELM == "true" ]]; then
	    check_previous_installation $TSLM_PKG_ELM_KEY
	fi
	if [[ $OPT_HDM == "true" ]]; then
	    check_previous_installation $TSLM_PKG_HDM_KEY
	fi
	if [[ $OPT_CLI == "true" ]]; then
	    check_previous_installation $TSLM_PKG_CLI_KEY
	fi
	if [[ $OPT_3494 == "true" ]]; then
	    check_previous_installation $TSLM_PKG_3494_KEY
	fi
	if [[ $OPT_PW == "true" ]]; then
	    check_previous_installation $TSLM_PKG_PWGEN_KEY
	fi
    fi
}

#
# check installed components
#
check_prerequisite()
{
    print_and_log_with_banner "Checking prerequisite"

    check_os_version || check_mode $?
    
    if [[ $OPT_3494 == "true" || $OPT_HDM == "true" || $OPT_CLI == "true" || $OPT_LB == "true" ]]; then
	if [[ $OPT_INSTALL == "true" ]]; then
	    check_perl || check_mode $?
	fi
    fi
    if [[ $OPT_LM == "true" || $OPT_HDM == "true" ]]; then
	if [[ $OPT_INSTALL == "true" ]]; then
	    check_tape_driver || check_mode $?
	fi
    fi
    if [[ $OPT_MM == "true" && $OPT_INSTALL == "true" ]]; then
	check_prereq_MM || check_mode $?
    fi
    if [[ $OPT_LM == "true" && $OPT_INSTALL == "true" ]]; then
	check_prereq_LM || check_mode $?
    fi
    if [[ $OPT_HDM == "true" && $OPT_INSTALL == "true" ]]; then
	check_prereq_HDM || check_mode $?
    fi
    if [[ $OPT_CLI == "true" && $OPT_INSTALL == "true" ]]; then
	check_prereq_CLI || check_mode $?
    fi
    if [[ ( $OPT_MM == "true" && $OPT_INSTALL == "true" ) || $OPT_DROPDB == "true" ]]; then
	check_database_instance || check_mode $?
	write_tslm_cfg
	read_tslm_cfg
    fi
    if [[ $OPT_ONLY_CHK_PREREQ == "true" ]]; then	
	exit 0
    fi
}

check_mode()
{
    if [[ $OPT_ONLY_CHK_PREREQ != "true" && $1 != 0 ]]; then	
        quit_install_uninstall $1
    fi
}

compare_vrmfs()
{
    VRMF_A=$1
    VRMF_B=$2
    logonly "VRMF_A=$VRMF_A, VRMF_B=$VRMF_B"
    
    CUR_A=`echo $VRMF_A | cut -f 1 -d .`
    CUR_B=`echo $VRMF_B | cut -f 1 -d .`
    logonly "CUR_A=$CUR_A, CUR_B=$CUR_B"
    
    REMAINS_A=`echo $VRMF_A | cut -f 2- -d . -s`
    REMAINS_B=`echo $VRMF_B | cut -f 2- -d . -s`
    logonly "REMAINS_A=$REMAINS_A, REMAINS_B=$REMAINS_B"
    
    if [ "$CUR_A" -lt "$CUR_B" ]; then
        return 2
    elif [ "$CUR_A" -gt "$CUR_B" ]; then
        return 1
    else
        [[ $REMAINS_A == "" ]] || [[ $REMAINS_B == "" ]] && return 0 
        compare_vrmfs $REMAINS_A $REMAINS_B
    fi
}

#
# value would be stored into $RETVALUE
# ask_value($msg, $default)
ask_value()
{
    local msg=$1
    local default=$2

    while [ true ];
    do
	if [[ $MACHINE_OS == "AIX" ]]; then
	    echo "Please enter $msg [$default]:\c"
	else
	    echo -n "Please enter $msg [$default]:"
	fi
	read value
	if [[ $value == "" ]]; then
	    value=$default
	fi

	if [[ $MACHINE_OS == "AIX" ]]; then
	    echo "$msg is \"$value\". OK? ('yes' or 'no') \c"
	else
	    echo -n "$msg is \"$value\". OK? ('yes' or 'no') "
	fi
	read yesno
	while [ true ];
	do
	    rc=`echo $yesno | sed 's/[Yy][Ee][Ss]/yes/'`
	    if [[ $rc == "yes" ]]; then
		yesno="yes"
	    fi
	    
	    rc=`echo $yesno | sed 's/[Yy]/y/'`
	    if [[ $rc == "y" ]]; then
		yesno="yes"
	    fi
	    
	    rc=`echo $yesno | sed 's/[Nn][Oo]/no/'`
	    if [[ $rc == "no" ]]; then
		yesno="no"
	    fi
	    
	    rc=`echo $yesno | sed 's/[Nn]/n/'`
	    if [[ $rc == "n" ]] ; then
		yesno="no"
	    fi
	    
	    if [[ $yesno == "no" ]]; then
		break
	    elif [[ $yesno == "yes" ]]; then
		RETVALUE=$value
		return
	    else
		if [[ $MACHINE_OS == "AIX" ]]; then
		    echo "$msg is \"$value\". OK? ('yes' or 'no') \c"
		else
		    echo -n "$msg is \"$value\". OK? ('yes' or 'no') "
		fi
		read yesno
	    fi
	done
    done
}


# perl
#
# PERL_NAME?
# PERL_FILESET?
# PERL_REQ_VRMF?
#
check_perl()
{
    print_and_log "Checking $PERL_NAME..."
    if [[ $MACHINE_OS == "AIX" ]]; then #AIX
	LSLPP_PERL_FILESET=`lslpp -l $PERL_FILESET_AIX* 2>&1`
	logonly "LSLPP_PERL_FILESET=$LSLPP_PERL_FILESET"
	
	IS_PL_INVALID="1"
	PERL_VRMF=`lslpp -l $PERL_FILESET_AIX* 2>&1 | grep COMMITTED | head -n 1 | awk '{print $2}'`
    else
	RPM_PERL_FILESET=`rpm -q $PERL_FILESET 2>&1`
	logonly "RPM_PERL_FILESET=$RPM_PERL_FILESET"
	
	IS_PL_INVALID="1"
	PERL_VRMF=`rpm -q $PERL_FILESET_LINUX 2>&1 | cut -f 2 -d -`
    fi

    if [ -z "$PERL_VRMF" ]; then
        print_and_log "No $PERL_NAME detected on this system."
    else
        print_and_log "$PERL_NAME version $PERL_VRMF detected on this system."
        PERL_VRMF="$PERL_VRMF.0"
        compare_vrmfs $PERL_VRMF $PERL_REQ_VRMF
        if [ $? -ne "2" ]; then
            IS_PL_INVALID="0"
        fi
    fi

    if [ "$IS_PL_INVALID" -eq "1" ]; then
        print_and_log "$PRODUCT_NAME requires $PERL_NAME version $PERL_REQ_VRMF or higher."
	return 13
    else
        print_and_log "Accepted"
    fi
    
    print_and_log ""

    return 0
}

#
# IBM tape driver
#
#TAPE_DRIVER_NAME_LINUX="lin_tape"
#TAPE_DRIVER_FILESET_LINUX="lin_tape"
#TAPE_DRIVER_REQ_VRMF_LINUX="1.74.0.0"
#TAPE_DRIVER_NAME_AIX="Atape"
#TAPE_DRIVER_FILESET_AIX="Atape.driver"
#TAPE_DRIVER_REQ_VRMF_AIX="12.5.2.0"
#
check_tape_driver()
{
    
    if [[ $MACHINE_OS == "AIX" ]]; then #AIX
	TAPE_DRIVER_NAME=$TAPE_DRIVER_NAME_AIX
	print_and_log "Checking $TAPE_DRIVER_NAME_AIX driver..."
	LSLPP_TAPE_DRIVER_FILESET=`lslpp -l $TAPE_DRIVER_FILESET_AIX* 2>&1`
	RET=$?
	logonly "LSLPP_TAPE_DRIVER_FILESET=$LSLPP_TAPE_DRIVER_FILESET"
	if [[ $RET == 0 ]]; then
	    IS_TD_INVALID="1"
	    TAPE_DRIVER_VRMF=`lslpp -l $TAPE_DRIVER_FILESET_AIX* 2>&1 | grep COMMITTED | head -n 1 | awk '{print $2}'`
	    TAPE_DRIVER_REQ_VRMF=$TAPE_DRIVER_REQ_VRMF_AIX
	else
	    IS_TD_INVALID="2"
	fi
    else
	TAPE_DRIVER_NAME=$TAPE_DRIVER_NAME_LINUX
	print_and_log "Checking $TAPE_DRIVER_NAME_LINUX driver..."
	RPM_TAPE_DRIVER_FILESET=`rpm -q $TAPE_DRIVER_FILESET_LINUX 2>&1`
	RET=$?
	logonly "RPM_TAPE_DRIVER_FILESET=$RPM_TAPE_DRIVER_FILESET"
	if [[ $RET == 0 ]]; then
	    IS_TD_INVALID="1"
	    TAPE_DRIVER_VRMF=`rpm -q $TAPE_DRIVER_FILESET_LINUX 2>&1 | cut -f 2 -d -`
	    TAPE_DRIVER_REQ_VRMF=$TAPE_DRIVER_REQ_VRMF_LINUX
	else
	    IS_TD_INVALID="2"
	fi
    fi

    if [[ $IS_TD_INVALID == 2 ]]; then
        print_and_log "$PRODUCT_NAME requires $TAPE_DRIVER_NAME driver installation."
        return 14
    fi	
    
    if [ -z "$TAPE_DRIVER_VRMF" ]; then
        print_and_log "No $TAPE_DRIVER_NAME driver detected on this system."
    else
        print_and_log "$TAPE_DRIVER_NAME driver version $TAPE_DRIVER_VRMF detected on this system."
        compare_vrmfs $TAPE_DRIVER_VRMF $TAPE_DRIVER_REQ_VRMF
        if [ $? -ne "2" ]; then
            IS_TD_INVALID="0"
        fi
    fi

    if [ "$IS_TD_INVALID" -eq "1" ]; then
        print_and_log "$PRODUCT_NAME requires $TAPE_DRIVER_NAME driver version $TAPE_DRIVER_REQ_VRMF or higher."
        return 14
    else
        print_and_log "Accepted"
    fi
    
    print_and_log ""
    return 0
}

#
# check database instance
#
#DB_ADM_NAME="db2inst1"
#DB_ADM_GRP_NAME="db2grp1"
check_database_instance()
{
    print_and_log "Checking $DB_NAME Instance..."
    
    value=""
    while [[ $value == "" ]]; do
	ask_value "DB2 instance user name" $DB_ADM_NAME
	DB_ADM_NAME=$RETVALUE
	value=`cat /etc/passwd | grep $DB_ADM_NAME`
	if [[ $value == "" ]]; then
	    ask_yes_no_answer "$DB_ADM_NAME is not created yet. Would you like to enter again? " && print_and_log "Aborted!" && exit 29
	fi
    done
    value=""
    while [[ $value == "" ]]; do
	ask_value "DB2 instance group name" $DB_ADM_GRP_NAME
	DB_ADM_GRP_NAME=$RETVALUE
	value=`cat /etc/group | grep $DB_ADM_GRP_NAME`
	if [[ $value == "" ]]; then
	    ask_yes_no_answer "$DB_ADM_GRP_NAME is not created yet. Would you like to enter again? " && print_and_log "Aborted!" && exit 29
	fi
    done
    value=""
    while [[ $value == "" ]]; do
	ask_value "DB2 instance directory" `cat /etc/passwd | grep $DB_ADM_NAME | cut -f6 -d':'`
	DB_ADM_DIR=$RETVALUE
	SQLLIB_PATH="$DB_ADM_DIR/sqllib/bin"
	if [[ ! -d $DB_ADM_DIR ]]; then
	    ask_yes_no_answer "$DB_ADM_DIR does not exist. Would you like to enter again? " && print_and_log "Aborted!" && exit 29
	    value=""
	elif [[ ! -d $SQLLIB_PATH ]]; then
	    ask_yes_no_answer "$SQLLIB_PATH does not exist. Would you like to enter again? " && print_and_log "Aborted!" && exit 29
	    value=""
	else
	    value="accepted"
	fi
    done

    grep "^$DB_ADM_NAME:" /etc/passwd > /dev/null
    GREP_DB_ADM_RES=$?
    if [ $GREP_DB_ADM_RES != 0 ]; then
        print_and_log "User \"$DB_ADM_NAME\" not found on this system."
        print_and_log "User \"$DB_ADM_NAME\" is required to create database for $PRODUCT_SHORT."
        return 15
    fi

    grep "^$DB_ADM_GRP_NAME:" /etc/group > /dev/null
    GREP_DB_GRP_RES=$?
    if [ $GREP_DB_GRP_RES != 0 ]; then
        print_and_log "Group \"$DB_ADM_GRP_NAME\" not found on this system."
        print_and_log "Group \"$DB_ADM_GRP_NAME\" is required to create database for $PRODUCT_SHORT."
        return 15
    else
        print_and_log "Accepted"
    fi

    print_and_log ""

    DB_VER=`su - $DB_ADM_NAME -c "db2licm -l" | grep Version | tr -d [:blank:] | cut -f2 -d\"`
    print_and_log "DB2 $DB_VER is detected"

    if [[ $DB_VER != "10.5" && $DB_VER != "9.7" ]]; then
	print_and_log "The DB2 version $DB_VER is not accepted"
	return 16
    elif [[ $DB_VER != "10.5" && $MACHINE_OS == "RHEL" && $MACHINE_VER == "7" ]]; then
	print_and_log "$MACHINE_OS $MACHINE_VER.$MACHINE_REL does not support DB2 $DB_VER"
	return 16
    else
	print_and_log "Accepted"
    fi

    print_and_log ""

    return 0
}

#
# check prerequired packages
#
check_prereq_linux()
{
    rpm -q $1
    RC=$?
    if [[ $RC != 0 ]]; then
	print_and_log "'$1' package is required for TSLM"
	print_and_log ""
	return 1
    fi
    return 0
}


check_prereq_MM()
{
    print_and_log ""
    print_and_log "Checking packages for Media Manager..."
    local RET=0
    if [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "6" ]]; then
	check_prereq_linux "audit-libs.i686" || RET=25
	check_prereq_linux "libstdc++.i686"  || RET=25
	check_prereq_linux "cracklib.i686" || RET=25
	check_prereq_linux "libselinux.i686" || RET=25
	check_prereq_linux "pam.i686" || RET=25
	check_prereq_linux "ksh.x86_64" || RET=25
	check_prereq_linux "db4.i686" || RET=25

    elif [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "7" ]]; then
	check_prereq_linux "audit-libs.i686" || RET=25
	check_prereq_linux "libstdc++.i686"  || RET=25
	check_prereq_linux "cracklib.i686" || RET=25
	check_prereq_linux "libselinux.i686" || RET=25
	check_prereq_linux "pam.i686" || RET=25
	check_prereq_linux "ksh.x86_64" || RET=25
    fi
    return $RET
}

check_prereq_LM()
{
    print_and_log ""
    print_and_log "Checking packages for Library Manager..."
    local RET=0
    if [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "6" ]]; then
	check_prereq_linux "sg3_utils.x86_64" || RET=26
	check_prereq_linux "sg3_utils-libs.x86_64" || RET=26
    elif [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "7" ]]; then
	check_prereq_linux "sg3_utils.x86_64" || RET=26
	check_prereq_linux "sg3_utils-libs.x86_64" || RET=26
    fi
    return $RET
}

check_prereq_HDM()
{
    print_and_log ""
    print_and_log "Checking packages for Host Drive Manager..."
    local RET=0
    if [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "6" ]]; then
	check_prereq_linux "sg3_utils.x86_64" || RET=27
	check_prereq_linux "sg3_utils-libs.x86_64" || RET=27
	check_prereq_linux "perl-Locale-Maketext-Simple" || RET=27
    elif [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "7" ]]; then
	check_prereq_linux "sg3_utils.x86_64" || RET=27
	check_prereq_linux "sg3_utils-libs.x86_64" || RET=27
	check_prereq_linux "perl-Locale-Maketext-Simple.noarch" || RET=27
	check_prereq_linux "perl-Locale-Maketext.noarch" || RET=27
    fi
    return $RET
}

check_prereq_CLI()
{
    print_and_log ""
    print_and_log "Checking packages for CLI..."
    local RET=0
    if [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "6" ]]; then
	check_prereq_linux "perl-Locale-Maketext-Simple" || RET=28
    elif [[ $MACHINE_OS == "RHEL" && $MACHINE_VER == "7" ]]; then
	check_prereq_linux "perl-Locale-Maketext-Simple.noarch" || RET=27
	check_prereq_linux "perl-Locale-Maketext.noarch" || RET=27
    fi
    return $RET
}

#
# do installation/uninstallation
#
do_install_uninstall()
{
    if [[ $OPT_INSTALL == "true" ]]; then
	print_and_log_with_banner "Start installing $PRODUCT_SHORT packages.."
	if [[ $OPT_SYS == "true" && $TSLM_SYS_INSTALLED == "false" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT System Package"
	    install_pkg $TSLM_PKG_SYSTEM_KEY
	elif [[ $OPT_SYS == "true" && $TSLM_SYS_INSTALLED == "true" ]]; then
	    print_and_log "$PRODUCT_SHORT System Package is installed already"
	fi
	if [[ $OPT_MM == "true" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT Media Manager Package"
	    install_pkg $TSLM_PKG_MM_KEY
	fi
	if [[ $OPT_LM == "true" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT Library Manager Package"
	    install_pkg $TSLM_PKG_LM_KEY
	fi
	if [[ $OPT_LB == "true" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT Library Balancer Package"
	    install_pkg $TSLM_PKG_LIBBAL_KEY
	fi
	if [[ $OPT_ELM == "true" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT External Library Manager Package"
	    install_pkg $TSLM_PKG_ELM_KEY
	fi
	if [[ $OPT_HDM == "true" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT Host Drive Manager Package"
	    install_pkg $TSLM_PKG_HDM_KEY
	fi
	if [[ $OPT_CLI == "true" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT CLI Package"
	    install_pkg $TSLM_PKG_CLI_KEY
	fi
	if [[ $OPT_3494 == "true" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT 3494 Emulator Package"
	    install_pkg $TSLM_PKG_3494_KEY
	fi
	if [[ $OPT_PW == "true" ]]; then
	    print_and_log "Installing $PRODUCT_SHORT Password Creation Tool (pwgen) Package"
	    install_pkg $TSLM_PKG_PWGEN_KEY
	fi

    elif [[ $OPT_UNINSTALL == "true" ]]; then
	print_and_log_with_banner "$PRODUCT_SHORT Uninstallation Menu"
	if [[ $OPT_LB == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT Library Balancer Package"
	    remove_pkg $TSLM_PKG_LIBBAL_KEY
	fi
	if [[ $OPT_ELM == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT External Library Manager Package"
	    remove_pkg $TSLM_PKG_ELM_KEY
	fi
	if [[ $OPT_HDM == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT Host Drive Manager Package"
	    kill_hdm_process
	    remove_pkg $TSLM_PKG_HDM_KEY
	fi
	if [[ $OPT_CLI == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT CLI Package"
	    remove_pkg $TSLM_PKG_CLI_KEY
	fi
	if [[ $OPT_3494 == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT 3494 Emulator Package"
	    remove_pkg $TSLM_PKG_3494_KEY
	fi
	if [[ $OPT_PW == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT Password Creation Tool (pwgen)  Package"
	    remove_pkg $TSLM_PKG_PWGEN_KEY
	fi
	if [[ $OPT_LM == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT Library Manager Package"
	    stop_lm_by_script
	    kill_lm_process
	    remove_pkg $TSLM_PKG_LM_KEY
	fi
	if [[ $OPT_MM == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT Media Manager Package"
	    stop_mm_by_script
	    kill_mm_process
	    remove_pkg $TSLM_PKG_MM_KEY
	fi
	if [[ $OPT_SYS == "true" ]]; then
	    print_and_log "Uninstalling $PRODUCT_SHORT System Package"
	    remove_pkg $TSLM_PKG_SYSTEM_KEY
	fi
	if [[ $OPT_DROPDB == "true" ]]; then
	    print_and_log ""
	    print_and_log "'$TSLM_DB_NAME' database will be dropped"
	fi
    fi
}

#
# post install/uninstall
#
post_install_uninstall()
{
    if [[ $OPT_INSTALL == "true" ]]; then
	if [[ $OPT_MM == "true" ]]; then
	    configure_db
	    prepare_mm_config
	fi
	if [[ $OPT_HDM == "true" ]]; then
	    prepare_hdm_config
	fi
	if [[ $OPT_ELM == "true" ]]; then
	    prepare_elm_config
	fi
	if [[ $OPT_3494 == "true" ]]; then
	    prepare_3494_config
	fi
    elif [[ $OPT_UNINSTALL == "true" ]]; then
	if [[ $OPT_DROPDB == "true" ]]; then
	    stop_lm_by_script
	    kill_lm_process
	    stop_mm_by_script
	    kill_mm_process
	    kill_hdm_process
	    su_check
	    drop_db
	fi
    fi
}

#
# database configuration
#
su_check()
{
    SU_PATH=`su - $TSLM_USR_NAME -c "env" 2>&1 | grep -e ^PATH=`
    logonly "SU_PATH=$SU_PATH"
    if [ `echo $SU_PATH | grep -c -e $SQLLIB_PATH` -ge "1" ]; then
        TSLM_SU_COMMAND="su - $TSLM_USR_NAME -c"
        logonly "TSLM_SU_COMMAND=$TSLM_SU_COMMAND"
        return
    fi

    SU_PATH=`su $TSLM_USR_NAME -c "env" 2>&1 | grep -e ^PATH=`
    logonly "SU_PATH=$SU_PATH"
    if [ `echo $SU_PATH | grep -c -e $SQLLIB_PATH` -ge "1" ]; then
        TSLM_SU_COMMAND="su $TSLM_USR_NAME -c"
        logonly "TSLM_SU_COMMAND=$TSLM_SU_COMMAND"
        return
    fi

    print_and_log "No directories are specified in the PATH of user $TSLM_USR_NAME to execute database commands."
    quit_install_uninstall 17
}

append_db_log()
{
    if [ -f $INSTALLATION_LOG_DIR/$DB2_INSTALL_LOG ]; then
        logonly ""
        logonly "--------------- Start of $INSTALLATION_LOG_DIR/$DB2_INSTALL_LOG ---------------"
        cat $INSTALLATION_LOG_DIR/$DB2_INSTALL_LOG >> $TSLM_INST_LOG_FILE_NAME
        logonly "---------------- End of $INSTALLATION_LOG_DIR/$DB2_INSTALL_LOG ----------------"
        logonly ""
    fi
}

configure_db()
{
    print_and_log "Creating database $TSLM_DB_NAME..."

    if [ ! -f $INSTALLATION_INSTALL_DIR/$PRE_INSTALL_DB_SCRIPT ]; then
        print_and_log "Script \"$PRE_INSTALL_DB_SCRIPT\" does not exist in $INSTALLATION_INSTALL_DIR/."
        print_and_log "Package installation seems to be failed."
	quit_install_uninstall 18
    fi
    
    if [ ! -f $INSTALLATION_INSTALL_DIR/$INSTALL_DB_SCRIPT ]; then
        print_and_log "Script \"$INSTALL_DB_SCRIPT\" does not exist in $INSTALLATION_INSTALL_DIR/."
        print_and_log "Package installation seems to be failed."
	quit_install_uninstall 18
    fi
    
    $INSTALLATION_INSTALL_DIR/$PRE_INSTALL_DB_SCRIPT
    
    su_check
    
    $TSLM_SU_COMMAND "$INSTALLATION_INSTALL_DIR/$INSTALL_DB_SCRIPT"
    CONFIGURE_DB_RC=$?
    append_db_log
    #echo "2" > $TSLM_INST_STAT_FILE_NAME
    if [ $CONFIGURE_DB_RC != 0 ]; then
        if [ $CONFIGURE_DB_RC == 4 ]; then
            ask_yes_no_answer "Would you like to recreate database $TSLM_DB_NAME?"
	    ASKRC=$?
            if [[ $ASKRC == "1" ]]; then
                ask_yes_no_answer "All data in the current database will be lost. Would you really like to continue?"
		ASKRC=$?
                if [[ $ASKRC == "1" ]]; then
                    $TSLM_SU_COMMAND "db2 connect reset" >> $TSLM_INST_LOG_FILE_NAME 2>&1
                    $TSLM_SU_COMMAND "db2 drop db $TSLM_DB_NAME" >> $TSLM_INST_LOG_FILE_NAME 2>&1
                    CONFIGURE_DB_RC=$?
                    if [ $CONFIGURE_DB_RC != 0 ]; then
                        print_and_log "Failed to drop $TSLM_DB_NAME DB. RC=$CONFIGURE_DB_RC"
			quit_install_uninstall 18
                    fi
                    #echo "1" > $TSLM_INST_STAT_FILE_NAME
                    $TSLM_SU_COMMAND "$INSTALLATION_INSTALL_DIR/$INSTALL_DB_SCRIPT"
                    CONFIGURE_DB_RC=$?
                    append_db_log
                    #echo "2" > $TSLM_INST_STAT_FILE_NAME
                    if [ $CONFIGURE_DB_RC != 0 ]; then
                        print_and_log "Failed to create $TSLM_DB_NAME DB. RC=$CONFIGURE_DB_RC"
			quit_install_uninstall 18
                    fi
                fi
            fi    
        else
            print_and_log "Failed to create $TSLM_DB_NAME DB. RC=$CONFIGURE_DB_RC"
	    quit_install_uninstall 18
        fi
    fi

    print_and_log "Done!"
    print_and_log ""
}

drop_db()
{
    print_and_log "Dropping database $TSLM_DB_NAME..."
    
    ask_yes_no_answer "All data in the current database will be lost. Would you really like to continue?"
    ASKRC=$?
    if [[ $ASKRC == "1" ]]; then
        $TSLM_SU_COMMAND "db2 connect reset" >> $TSLM_UNINST_LOG_FILE_NAME 2>&1
        $TSLM_SU_COMMAND "db2 drop db $TSLM_DB_NAME" >> $TSLM_UNINST_LOG_FILE_NAME 2>&1
        DROP_DB_RC=$?
        if [ $DROP_DB_RC -ne 0 ]; then
            print_and_log "Failed to drop $TSLM_DB_NAME database. RC=$DROP_DB_RC"
            ask_yes_no_answer "Would you like to continue anyway?"
	    ASKRC=$?
            if [ $ASKRC == "0" ]; then
                quit_install_uninstall 19
            fi
        else
            print_and_log "Done!"
        fi
    else
        print_and_log "Dropping database was skipped by user's request."
    fi
    
    print_and_log ""
}

#
# config files
#
copy_tslm_file()
{
    cp $1 $2 >> $TSLM_INST_LOG_FILE_NAME 2>&1
    TSLM_INST_CP_CONFIG_RES=$?
    if [ $TSLM_INST_CP_CONFIG_RES != 0 ]; then
        print_and_log "Failed to copy \"$1\" to \"$2\": RC=$TSLM_INST_CP_CONFIG_RES"
	quit_install_uninstall 20
    fi
}

prepare_mm_config()
{
    print_and_log "Preparing $TSLM_MM config file..."
    if [ ! -f $INSTALLATION_CONFIG_DIR/$MM_CONFIG_FILE_SAMPLE ]; then
        print_and_log "File \"$MM_CONFIG_FILE_SAMPLE\" does not exist in $INSTALLATION_CONFIG_DIR/."
        print_and_log "Package installation seems to be failed."
        quit_install_uninstall 21
    else
        if [ -f $INSTALLATION_CONFIG_DIR/$MM_CONFIG_FILE ]; then
            print_and_log "Config file of $TSLM_MM \"$MM_CONFIG_FILE\" already exists in $INSTALLATION_CONFIG_DIR/."
            ask_yes_no_answer "Would you like to use the current config file?"
	    ASKRC=$?
            if [[ $ASKRC == "1" ]]; then
                copy_tslm_file $INSTALLATION_CONFIG_DIR/$MM_CONFIG_FILE_SAMPLE $INSTALLATION_CONFIG_DIR/$MM_CONFIG_FILE
            fi
        else
            copy_tslm_file $INSTALLATION_CONFIG_DIR/$MM_CONFIG_FILE_SAMPLE $INSTALLATION_CONFIG_DIR/$MM_CONFIG_FILE
        fi
    fi

    print_and_log "Done!"
    print_and_log ""
}

prepare_hdm_config()
{
    print_and_log "Preparing $TSLM_HDM config file..."
    if [ ! -f $INSTALLATION_CONFIG_DIR/$HDM_CONFIG_FILE_SAMPLE ]; then
        print_and_log "File \"$HDM_CONFIG_FILE_SAMPLE\" does not exist in $INSTALLATION_CONFIG_DIR/."
        print_and_log "Package installation seems to be failed."
        quit_install_uninstall 22
    else
        if [ -f $INSTALLATION_CONFIG_DIR/$HDM_CONFIG_FILE ]; then
            print_and_log "Config file of $TSLM_HDM \"$HDM_CONFIG_FILE\" already exists in $INSTALLATION_CONFIG_DIR/."
            ask_yes_no_answer "Would you like to use the current config file?"
	    ASKRC=$?
            if [[ $? == "0" ]]; then
                copy_tslm_file $INSTALLATION_CONFIG_DIR/$HDM_CONFIG_FILE_SAMPLE $INSTALLATION_CONFIG_DIR/$HDM_CONFIG_FILE
            fi
        else
            copy_tslm_file $INSTALLATION_CONFIG_DIR/$HDM_CONFIG_FILE_SAMPLE $INSTALLATION_CONFIG_DIR/$HDM_CONFIG_FILE
        fi
    fi
    print_and_log "Done!"
    print_and_log ""
}

prepare_elm_config()
{
    print_and_log "Preparing $TSLM_ELM config file..."
    if [ ! -f $INSTALLATION_TSM_DIR/$ELM_CONFIG_FILE_SAMPLE ]; then
        print_and_log "File \"$ELM_CONFIG_FILE_SAMPLE\" does not exist in $INSTALLATION_TSM_DIR/."
        print_and_log "Package installation seems to be failed."
        quit_install_uninstall 22
    else
        if [ -f $INSTALLATION_TSM_DIR/$ELM_CONFIG_FILE ]; then
            print_and_log "Config file of $TSLM_ELM \"$ELM_CONFIG_FILE\" already exists in $INSTALLATION_TSM_DIR/."
            ask_yes_no_answer "Would you like to use the current config file?"
	    ASKRC=$?
            if [[ $ASKRC == "0" ]]; then
                copy_tslm_file $INSTALLATION_TSM_DIR/$ELM_CONFIG_FILE_SAMPLE $INSTALLATION_TSM_DIR/$ELM_CONFIG_FILE
            fi
        else
            copy_tslm_file $INSTALLATION_TSM_DIR/$ELM_CONFIG_FILE_SAMPLE $INSTALLATION_TSM_DIR/$ELM_CONFIG_FILE
        fi
    fi
    
    print_and_log "Done!"
    print_and_log ""
}

prepare_3494_config()
{
    print_and_log "Preparing $TSLM_EMU3494 config file..."
    if [ ! -f $INSTALLATION_EMU3494_DIR/$EMU3494_CONFIG_FILE_SAMPLE ]; then
        print_and_log "File \"$EMU3494_CONFIG_FILE_SAMPLE\" does not exist in $INSTALLATION_EMU3494_DIR/."
        print_and_log "Package installation seems to be failed."
        quit_install_uninstall 23
    else
        if [ -f $INSTALLATION_EMU3494_DIR/$EMU3494_CONFIG_FILE ]; then
            print_and_log "Config file of $TSLM_EMU3494 \"$EMU3494_CONFIG_FILE\" already exists in $INSTALLATION_EMU3494_DIR/."
            ask_yes_no_answer "Would you like to use the current config file?"
	    ASKRC=$?
            if [[ $ASKRC == "0" ]]; then
                copy_tslm_file $INSTALLATION_EMU3494_DIR/$EMU3494_CONFIG_FILE_SAMPLE $INSTALLATION_EMU3494_DIR/$EMU3494_CONFIG_FILE
            fi
        else
            copy_tslm_file $INSTALLATION_EMU3494_DIR/$EMU3494_CONFIG_FILE_SAMPLE $INSTALLATION_EMU3494_DIR/$EMU3494_CONFIG_FILE
        fi
    fi

    print_and_log "Done!"
    print_and_log ""
}

#
# TSLM configuration file
#
write_tslm_cfg()
{
    if [[ -e $TSLM_CONF_FILE ]]; then
	print_and_log "Deleting $TSLM_CONF_FILE..."
	rm -f $TSLM_CONF_FILE
    fi

    print_and_log "Creating $TSLM_CONF_FILE..."
    echo "DB_ADM_NAME=$DB_ADM_NAME" > $TSLM_CONF_FILE
    echo "DB_ADM_GRP_NAME=$DB_ADM_GRP_NAME" >> $TSLM_CONF_FILE
    echo "DB_ADM_DIR=$DB_ADM_DIR" >> $TSLM_CONF_FILE
    echo "SQLLIB_PATH=$SQLLIB_PATH" >> $TSLM_CONF_FILE
}


find_value_tslm_cfg()
{
    value=`cat $TSLM_CONF_FILE | grep $1 | tr -d ' ' | cut -f2 -d'='`
    echo $value
}

read_tslm_cfg() # for testing only
{
    if [[ -e $TSLM_CONF_FILE ]]; then
	print_and_log "Reading $TSLM_CONF_FILE..."
	
	for target in "DB_ADM_NAME" "DB_ADM_GRP_NAME" "DB_ADM_DIR" "SQLLIB_PATH" "HOGE"
	do
	    line=`find_value_tslm_cfg $target`
	    if [[ $line != "" ]]; then
		eval $target=$line
	    else
		case "$target" in
		    "DB_ADM_NAME") DB_ADM_NAME="db2inst1" ;;
		    "DB_ADM_GRP_NAME") DB_ADM_GRP_NAME="db2grp1" ;;
		    "DB_ADM_DIR") DB_ADM_DIR="/home/db2inst1" ;;
		    "SQLLIB_PATH") SQLLIB_PATH="/home/db2inst1/sqllib/bin" ;;
		esac
	    fi
	done
    else
	DB_ADM_NAME="db2inst1"
	DB_ADM_GRP_NAME="db2grp1"
	DB_ADM_DIR="/home/db2inst1"
	SQLLIB_PATH="/home/db2inst/sqllib/bin"
    fi
}


########################################################################
# main
########################################################################
show_copyright
check_user
create_log_file
OPTIND=1
check_platform
check_options $@
print_and_log_with_banner "Start of $PRODUCT_SHORT installation process on `date`"
print_and_log ""
usage 0
check_valid_options
check_prerequisite
show_action_menu
set_package_dir
check_previous_installations
do_install_uninstall
post_install_uninstall

