#!/bin/sh
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2016 All Rights Reserved
#
# US Government Users Restricted Rights - Use,  duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-----------------------------------------------------------------------
#
# @version 1.5 - 16/06/19 22:54:55
#
# @brief
#
#  The following three lines have been added by UDB DB2.
#
#-----------------------------------------------------------------------

echo "IBM Tape System Library Manager"
echo "Licensed Materials - Property of IBM"
echo "(C) Copyright IBM Corp. 2004, 2016 All Rights Reserved"
echo


# This script must run by ermm user
if [ "$USER" != "ermm" ]; then
    echo "This script must be run with ermm account."
    exit 1
fi


# Log file for this script 
LOG=/opt/IBM/ermm/log/tslmsetlogarchive.log


# check if the Media Manager is running 
MMRUN=$(ps -ef | awk '{print $8}' | grep "^/opt/IBM/ermm/bin/MediaManager" )
if [ ! -z "$MMRUN" ]
then
    echo "Stop the Media Manager before changing database setting" 
    exit 2
fi

if [ -f $INSTHOME/sqllib/db2profile ]; then
    . $INSTHOME/sqllib/db2profile
fi

# Get definitions of backup setting
. /opt/IBM/ermm/conf/dbuserexit.conf

# TSLMAUDITERRPATH   : archive log erro path
# TSLMDBLOGARCHIVE   : directory of archive log files
# TSLMDBBACKUPDIR    : directory of db backup file
# TSLMDBLOGMIRRORDIR : directory of mirror log files
# Check those directories can be accessed
touch $TSLMAUDITERRPATH/tslmcheck
if [ $? != 0 ]
then
    echo "Cannot access to $TSLMAUDITERRPATH directory, please check the access permission." 
    echo "Please run tslmmkarchive.sh with root user to make directories for backup."
    exit 2
fi
rm $TSLMAUDITERRPATH/tslmcheck
touch $TSLMDBLOGARCHIVE/tslmcheck
if [ $? != 0 ]
then
    echo "Cannot access to $TSLMDBLOGARCHIVE directory, please check the access permission." 
    echo "Please run tslmmkarchive.sh with root user to make directories for backup."    
    exit 2
fi
rm $TSLMDBLOGARCHIVE/tslmcheck
touch $TSLMDBBACKUPDIR/tslmcheck
if [ $? != 0 ]
then
    echo "Cannot access to $TSLMDBBACKUPDIR directory, please check the access permission." 
    echo "Please run tslmmkarchive.sh with root user to make directories for backup."    
    exit 2
fi
rm $TSLMDBBACKUPDIR/tslmcheck
touch $TSLMDBLOGMIRRORDIR/tslmcheck
if [ $? != 0 ]
then
    echo "Cannot access to $TSLMDBLOGMIRRORDIR directory, please check the access permission." 
    echo "Please run tslmmkarchive.sh with root user to make directories for backup."    
    exit 2
fi
rm $TSLMDBLOGMIRRORDIR/tslmcheck

#
# Update db cfg parameter mirrorlogpath
#
echo "Updating MIRRORLOGPATH parameter ..." | tee  $LOG
db2 "update db cfg for ermm using MIRRORLOGPATH $TSLMDBLOGMIRRORDIR" | tee -a $LOG
if [ $? != 0 ]
then
    echo "Cannot update MIRRORLOGPATH parameter of database ermm." | tee -a $LOG
    exit 2
fi

#
# Update db cfg parameter logarchmeth1 select LOGRETAIN or USEREXIT
#


echo "Setting LOGARCHMETH1 USEREXIT" | tee -a $LOG
db2 "update db cfg for ermm using LOGARCHMETH1 USEREXIT" | tee -a $LOG 2>&1 
if [ $? != 0 ]
then
    echo "Cannot update LOGARCHMETH1 for database ermm." | tee -a $LOG
    exit 2
fi

#
# backup of ermm database
#
echo "Making first backup of ermm database ..." | tee -a $LOG
db2 "backup database ermm to $TSLMDBBACKUPDIR" | tee -a $LOG 2>&1
if [ $? != 0 ]
then
    echo "Cannot backup database ermm." \ tee -a $LOG
    exit 2
fi

echo "Successfully changing the database setting to archive logging mode."
exit 0
